@@ -17,13 +17,13 @@ class BackupGenerator(Log):
         if not os.path.exists(dir_name):
             os.mkdir(dir_name)
 
-        oc = self._config.create_oc()
+        oc = self._config.create_api()
         namespaces = oc.get_namespaces()
         apis = oc._exec(['api-resources', '--namespaced', '-o', 'name']).splitlines()
         for namespace in namespaces:
             namespace = namespace.split('/')[1]
             self.log.info(f'Backup up namespace {namespace}')
-            oc.project(namespace)
+            oc.set_namespace(namespace)
             for api in apis:
                 try:
                     names = oc._exec(['get', api, '-o', 'name']).splitlines()
