@@ -7,9 +7,11 @@ import yaml
 
 from octoploy.config.Config import ProjectConfig, AppConfig, RunMode
 from octoploy.deploy.DeploymentBundle import DeploymentBundle
-from octoploy.deploy.OcObjectDeployer import OcObjectDeployer
+from octoploy.deploy.K8sObjectDeployer import K8sObjectDeployer
 from octoploy.processing.YmlTemplateProcessor import YmlTemplateProcessor
+from octoploy.utils.Errors import SkipObject
 from octoploy.utils.Log import Log
+from octoploy.utils.Yml import Yml
 
 
 class AppDeployment:
@@ -78,34 +80,45 @@ class AppDeployRunner(Log):
         if self._app_config.is_template():
             raise ValueError('App is a template and can\'t be deployed')
 
+        # Resolve all templating references
         template_processor = self._app_config.get_template_processor()
         template_processor.parent(self._root_config.get_template_processor())
 
-        self._deploy_templates(self._app_config.get_pre_template_refs(), template_processor)
+        self._apply_templates(self._app_config.get_pre_template_refs(), template_processor)
         self._load_files(self._app_config.get_config_root(), template_processor)
-        self._deploy_extra_configmaps(template_processor)
-        self._deploy_templates(self._app_config.get_post_template_refs(), template_processor)
+        self._load_extra_configmaps(template_processor)
+        self._apply_templates(self._app_config.get_post_template_refs(), template_processor)
 
-        k8api = self._root_config.create_oc()
+        # Additional processing
+        tree_processors = self._root_config.get_yml_processors()
+        for processor in tree_processors:
+            for data in list(self._bundle.objects):
+                try:
+                    processor.process(data)
+                except SkipObject as e:
+                    self.log.warning(f'Skipping object: {e}')
+                    self._bundle.objects.remove(data)
+
+        k8sapi = self._root_config.create_api()
         if self._mode.out_file is not None:
             self._bundle.dump_objects(self._mode.out_file)
         if self._mode.dry_run:
             return
 
-        self.log.info('Checking ' + self._app_config.get_dc_name())
-        object_deployer = OcObjectDeployer(self._root_config, k8api, self._app_config, mode=self._mode)
+        self.log.info(f'Checking {self._app_config.get_dc_name()}')
+        object_deployer = K8sObjectDeployer(self._root_config, k8sapi, self._app_config, mode=self._mode)
         self._bundle.deploy(object_deployer)
 
-    def _deploy_templates(self, template_names: List[str], template_processor: YmlTemplateProcessor):
+    def _apply_templates(self, template_names: List[str], template_processor: YmlTemplateProcessor):
         """
         Deploys all referenced templates (recursively)
         """
         for template_name in template_names:
             template = self._root_config.load_app_config(template_name)
             if not template.is_template():
-                raise ValueError('Referenced app ' + template_name + ' is not declared as template')
+                raise ValueError(f'Referenced app {template_name} is not declared as template')
             if not template.enabled():
-                self.log.warning('Template ' + template_name + ' is disabled, skipping')
+                self.log.warning(f'Template {template_name} is disabled, skipping')
                 return
 
             child_template_processor = YmlTemplateProcessor(template)
@@ -116,9 +129,9 @@ class AppDeployRunner(Log):
 
             # The template might reference other templates
             # -> Recursively deploy them
-            self._deploy_templates(template.get_pre_template_refs(), child_template_processor)
+            self._apply_templates(template.get_pre_template_refs(), child_template_processor)
             self._load_files(template.get_config_root(), child_template_processor)
-            self._deploy_templates(template.get_post_template_refs(), child_template_processor)
+            self._apply_templates(template.get_post_template_refs(), child_template_processor)
 
     def _load_files(self, root: str, template_processor: YmlTemplateProcessor):
         """
@@ -129,22 +142,17 @@ class AppDeployRunner(Log):
             path = os.path.join(root, item)
             if not os.path.isfile(path) or not item.endswith('.yml') or item.startswith('_'):
                 continue
+            try:
+                docs = Yml.load_docs(path)
+            except yaml.parser.ParserError as e:
+                self.log.error(f'Could not parse {path} {e}')
+                raise
+            for doc in docs:
+                self._bundle.add_object(doc, template_processor)
 
-            with open(path, 'r') as stream:
-                try:
-                    data = yaml.load_all(stream, Loader=yaml.FullLoader)
-                    for doc in data:
-                        if doc is None:
-                            # Empty block
-                            continue
-                        self._bundle.add_object(doc, template_processor)
-                except yaml.parser.ParserError as e:
-                    print(f'Could not parse {path} {e}')
-                    raise
-
-    def _deploy_extra_configmaps(self, template_processor: YmlTemplateProcessor):
+    def _load_extra_configmaps(self, template_processor: YmlTemplateProcessor):
         """
-        Deploys all defined file based configmaps
+        Loads all defined file based configmaps
         :param template_processor: Template processor which should be applied
         """
         for config in self._app_config.get_config_maps():
